==> swi-prolog_8.2.4+dfsg.orig.tar.gz: swipl-8.2.4+dfsg/src/Tests/library/test_lazy_lists.pl <==
/*  Part of SWI-Prolog

    Author:        Jan Wielemaker
    E-mail:        J.Wielemaker@vu.nl
    WWW:           http://www.swi-prolog.org
    Copyright (c)  2016, University of Amsterdam
                         CWI, Amsterdam
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:

    1. Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.

    2. Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in
       the documentation and/or other materials provided with the
       distribution.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
    POSSIBILITY OF SUCH DAMAGE.
*/

:- module(test_lazy_lists,
	  [ test_lazy_lists/0
	  ]).
:- use_module(library(lazy_lists)).
:- use_module(library(plunit)).
:- use_module(library(dcg/basics)).

test_lazy_lists :-
	run_tests([ lazy_lists
		  ]).

:- begin_tests(lazy_lists).

test(list, Rest == `world`) :-
	setup_call_cleanup(
	    ( open_string("hello world", In),
	      lazy_list(lazy_get_codes(In, 1), List)
	    ),
	    phrase(("hello", " ", string(Rest)), List),
	    close(In)),
	!.

:- end_tests(lazy_lists).

==> swi-prolog_9.0.4+dfsg.orig.tar.gz: swipl-9.0.4+dfsg/src/Tests/library/test_lazy_lists.pl <==
/*  Part of SWI-Prolog

    Author:        Jan Wielemaker
    E-mail:        J.Wielemaker@vu.nl
    WWW:           http://www.swi-prolog.org
    Copyright (c)  2016, University of Amsterdam
                         CWI, Amsterdam
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:

    1. Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.

    2. Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in
       the documentation and/or other materials provided with the
       distribution.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
    POSSIBILITY OF SUCH DAMAGE.
*/

:- module(test_lazy_lists,
	  [ test_lazy_lists/0
	  ]).
:- use_module(library(lazy_lists)).
:- use_module(library(plunit)).
:- use_module(library(dcg/basics)).

test_lazy_lists :-
	run_tests([ lazy_lists
		  ]).

:- begin_tests(lazy_lists).

test(list, Rest == `world`) :-
	setup_call_cleanup(
	    ( open_string("hello world", In),
	      lazy_list(lazy_get_codes(In, 1), List)
	    ),
	    phrase(("hello", " ", string(Rest)), List),
	    close(In)),
	!.

% Verify that the Next argument is not copied

test(state, Final == 5) :-
    State = x(1, 5),
    lazy_list(next(State), List),
    forall(member(_, List),
           true),
    arg(1, State, Final).

next(State, List, Tail) :-
    State = x(Here, Max),
    (   Here < Max
    ->  Here2 is Here+1,
        List = [Here2|Tail],
        nb_setarg(1, State, Here2)
    ;   List = [],
        Tail = []
    ).

:- end_tests(lazy_lists).
